
#include "TArtAnaFile.hh"
 
#include <iostream>
#include <iomanip>
#include <vector>
#include <map>
#include <fstream>
#include <sstream>
#include <stdexcept>
#include <algorithm>
#include <cstdlib>

#include "TH1.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TCutG.h"
#include "TPad.h"

#include "TArtAnaClock.hh"

extern TH1* gH1;

TArtValMap::TArtValMap()
{
  memset(fFlag,false,sizeof(bool)*MaxAnalyserSize*MaxWNumSize*MaxIndexSize);
}

TArtValMap::~TArtValMap()
{;}

void TArtValMap::Clear()
{
  for(int i=0; i<(int)fCleanList.size(); ++i){
    int analyser = fCleanList[i].fAnalyser;
    int wnum = fCleanList[i].fWNum;
    int index = fCleanList[i].fIndex;
    fFlag[analyser][wnum][index] = false;
  }
  fCleanList.clear();
}

bool TArtAnaFile::Construct()
{
  if(!AddSequenceMap("hist", &TArtAnaFile::Hist)) return false;
  if(!AddSequenceMap("and", &TArtAnaFile::And)) return false;
  if(!AddSequenceMap("or", &TArtAnaFile::Or)) return false;
  if(!AddSequenceMap("gate", &TArtAnaFile::Gate)) return false;
  if(!AddSequenceMap("cc", &TArtAnaFile::CC)) return false;
  if(!AddSequenceMap("cutg", &TArtAnaFile::CutG)) return false;
  if(!AddSequenceMap("point", &TArtAnaFile::Point)) return false;
  if(!AddSequenceMap("include", &TArtAnaFile::Include)) return false;

  return true;
}

bool TArtAnaFile::Open(const char* filename)
{
  std::ifstream ifs;
  ifs.open(filename, std::ios::in);
  if(!ifs){
    std::cout << "cannot open file" << std::endl;
    return false;
  }

  int countline=0;
  while(1){
    if(ifs.eof()) break;
    std::string str;
    std::getline(ifs, str);
    ++countline;
    if(str.empty()) continue;
    if(str.find("#") != std::string::npos) continue;

    //search sequence
    std::string sequence;
    if(!ReadSequence(str, countline, sequence)) return false;
    if(sequence.empty()) sequence = "hist";

    std::map<std::string, FSequence>::iterator it = fSequenceMap.find(sequence);
    if(it == fSequenceMap.end()){
      std::cout << "sequence " << sequence << " does not defined" << std::endl;
      return false;
    }else{
      if(!(this->*(it->second))(str, countline)) return false;
    }
  }

  return true;
}

bool TArtAnaFile::Check()
{
  bool exist = true;
  for(int i=0; i<(int)fH1Array.size(); ++i){
    TH1* hist = fH1Array[i];
    std::string name = hist->GetName();
    int gate = fGateMap[name];

    if(gate == 0) continue;
    std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.find(gate);
    if(it == fAnaGateMap.end()){
      std::cout << "gate: " << gate << " does not exist." << std::endl;
      exist = false;
    }
  }
  return exist;
}

void TArtAnaFile::ClearVal()
{
  fValMap.Clear();
}

void TArtAnaFile::ClearMap()
{
  fH1Array.clear();
  fValIndexMap.clear();
  fGateMap.clear();
  fAnaGateMap.clear();
  fValIndexArrayMap.clear();
  fCutGMap.clear();

  fSequenceMap.clear();
}

bool TArtAnaFile::Add(int analyser, int wnum, int index, Double_t val)
{
  if(TArtValMap::MaxAnalyserSize <= analyser ||
     TArtValMap::MaxWNumSize <= wnum ||
     TArtValMap::MaxIndexSize <= index){
    TArtValIndex valindex(analyser, wnum, index);
    std::cout << "define larger memory size for val!!" << std::endl;
    std::cout << "valindex: " << valindex << std::endl;
    abort();
  }

  bool exist = fValMap.fFlag[analyser][wnum][index];
  if(!exist){
    fValMap.fFlag[analyser][wnum][index] = true;
    fValMap.fVal[analyser][wnum][index] = val;
    fValMap.fCleanList.push_back(TArtValIndex(analyser, wnum, index));
    return true;
  }else{
    TArtValIndex valindex(analyser, wnum, index);
    std::cout << "already exsits: " << valindex << std::endl;
    return false;
  }
}

void TArtAnaFile::Fill()
{
  std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.begin();
  while(it != fAnaGateMap.end()){
    CalGate(it->first);
    ++it;
  }

  //  TArtAnaClock::Start();

  for(int i=0; i<(int)fH1Array.size(); ++i){
    TH1* hist = fH1Array[i];
    std::string name = hist->GetName();
    int gate = fGateMap[name];
    if(!IsInside(gate)) continue;
    std::vector<TArtValIndex> valindexarray = fValIndexMap[name];
    if(hist->GetDimension() == 1){
      for(int j=0; j<(int)valindexarray.size(); ++j){
	Double_t val;
	if(Find(valindexarray[j], val)) hist->Fill(val);
      }
    }else if(hist->GetDimension() == 2){
      for(int j=0; j<(int)valindexarray.size(); j+=2){
	Double_t valx, valy;
	if(Find(valindexarray[j], valx) && Find(valindexarray[j+1], valy)) hist->Fill(valx, valy);
      }
    }
  }

//  TArtAnaClock::Stop();
//  std::cout << "\n time:" << std::endl;
//  TArtAnaClock::Print();
}

bool TArtAnaFile::CalGate(int gate)
{
  if(gate == 0) return true;

  TArtAnaGate& anagate = fAnaGateMap[gate];
  bool inside = false;
  if(anagate.fUL){
    std::vector<TArtValIndex> valindexarray = fValIndexArrayMap[gate];
    for(int j=0; j<(int)valindexarray.size(); ++j){
      Double_t val;
      if(Find(valindexarray[j], val)){
	if(anagate.fLow < val && anagate.fUp > val) inside = true;
      }
    }
  }else if(anagate.fAnd){
    for(int i=0; i<(int)anagate.fGateArray.size(); ++i){
      int gate_next = anagate.fGateArray[i];
      inside = inside && CalGate(gate_next);
    }
  }else if(anagate.fOr){
    for(int i=0; i<(int)anagate.fGateArray.size(); ++i){
      int gate_next = anagate.fGateArray[i];
      inside = inside || CalGate(gate_next);
    }
  }else if(anagate.fCutG){
    std::vector<TArtValIndex> valindexarray = fValIndexArrayMap[gate];
    for(int j=0; j<(int)valindexarray.size(); j+=2){
      Double_t valx, valy;
      if(Find(valindexarray[j], valx) &&
	 Find(valindexarray[j+1], valy)){
	if(anagate.fCutG->IsInside(valx, valy)) inside = true;
      }
    }
  }else if(anagate.fCC){
    std::vector<TArtValIndex> valindexarray = fValIndexArrayMap[gate];
    for(int j=0; j<(int)valindexarray.size(); j+=2){
      Double_t valx, valy;
      if(Find(valindexarray[j], valx) &&
	 Find(valindexarray[j+1], valy)){
	valx -= anagate.fCenterX;
	valx /= anagate.fRadiusX;
	valy -= anagate.fCenterY;
	valy /= anagate.fRadiusY;
	if(valx*valx+valy*valy < 1) inside = true;
      }
    }
  }
  return anagate.fInside = (gate > 0) ? inside : !inside;
}

bool TArtAnaFile::IsInside(int gate)
{
  if(gate == 0) return true;
  else return fAnaGateMap[gate].fInside;
}

bool TArtAnaFile::Find(TArtValIndex valindex, Double_t& val)
{
  int analyser = valindex.fAnalyser;
  int wnum = valindex.fWNum;
  int index = valindex.fIndex;
  bool exist = fValMap.fFlag[analyser][wnum][index];
  if(exist){
    val = fValMap.fVal[analyser][wnum][index];
    return true;
  }else{
    return false;
  }
}

TCutG* TArtAnaFile::CreateCutG(int id, const char* filename, const char* cutname)
{
  TH1* hist = gH1;
  if(!hist) return 0;
  if(hist->GetDimension() != 2){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }
  TH2* hist2 = static_cast<TH2*>(hist);

  if(!gPad) return 0;

  std::ofstream ofs;
  if(filename != 0){
    ofs.open(filename, std::ios::app);
  }else{
    std::cout << "write cut data to cut/temp.cut" << std::endl;
    filename = "cut/temp.cut";
    ofs.open(filename, std::ios::app);
  }
  if(!ofs){
    std::cout << "cannot open " << filename << std::endl;
    return 0;
  }

  std::map<std::string, std::vector<TArtValIndex> >::iterator it = fValIndexMap.find(hist2->GetName());
  if(it == fValIndexMap.end()){
    std::cout << hist2->GetName() << " does not exist" << std::endl;
    return 0;
  }

  if(id == 0){
    id = 1001;
    std::cout << "id is set to " << id << std::endl;
  }

  TCutG* cutg = static_cast<TCutG*>(gPad->WaitPrimitive("CUTG","CutG"));
  if(!cutg) return 0;
  if(cutname != 0){
    cutg->SetName(cutname);
  }else{
    cutg->SetName("cutg");    
  }

  if(cutg->GetN()<4){
    std::cout << "num of points must be larger than 3" << std::endl;
    return 0;
  }

  std::vector<TArtValIndex> valindexforgatemap = it->second;
  TArtValIndex valindexx = valindexforgatemap[0];
  int analyserx = valindexx.fAnalyser;
  int wnumx = valindexx.fWNum;
  int indexbx = valindexx.fIndex;
  int indexex = valindexx.fIndex;

  TArtValIndex valindexy = valindexforgatemap[1];
  int analysery = valindexy.fAnalyser;
  int wnumy = valindexy.fWNum;
  int indexby = valindexy.fIndex;
  int indexey = valindexy.fIndex;

  for(int i=2; i<(int)valindexforgatemap.size(); i+=2){
    indexex = valindexforgatemap[i].fIndex;
    indexey = valindexforgatemap[i+1].fIndex;
  }

  ofs << std::endl;
  ofs << "<cutg>  " << id << ", "
      << analyserx << ","
      << indexbx << ","
      << indexex << ","
      << wnumx << ", "
      << analysery << ","
      << indexby << ","
      << indexey << ","
      << wnumy << ", "
      << "\"" << cutg->GetName() << "\""
      << std::endl;
  for(int i=0; i<cutg->GetN(); ++i){
    Double_t x, y;
    cutg->GetPoint(i, x, y);
    ofs << "<point> " << id << ", "
	<< std::setw(10) << x << ", "
	<< std::setw(10) << y << ","
	<< std::endl;
  }

  return cutg;
}

bool TArtAnaFile::AddValIndexMap(std::string name, std::vector<TArtValIndex> valindexarray)
{
  std::map<std::string, std::vector<TArtValIndex> >::iterator it = fValIndexMap.find(name);
  if(it == fValIndexMap.end()){
    fValIndexMap.insert(std::pair<std::string, std::vector<TArtValIndex> >(name, valindexarray));
    return true;
  }else{
    std::cout << "conflict hist name: "<< name << std::endl;
    return false;
  }
}

bool TArtAnaFile::AddGateMap(std::string name, int gate)
{
  std::pair<std::map<std::string, int>::iterator, bool> ret
    = fGateMap.insert(std::pair<std::string, int>(name, gate));
  if(!ret.second){
    std::cout << "conflict hist name: "<< name << std::endl;
    return false;
  }
  return true;
}
 
bool TArtAnaFile::AddValIndexArrayMap(int gate, std::vector<TArtValIndex> valindexarray)
{
  if(gate == 0){
    std::cout << "cannot use gate number 0" << std::endl;
    return false;
  }

  std::pair<std::map<int, std::vector<TArtValIndex> >::iterator, bool> ret
    = fValIndexArrayMap.insert(std::pair<int, std::vector<TArtValIndex> >(gate, valindexarray));
  if(!ret.second){
    std::cout << "conflict gate number: "<< gate << std::endl;
    return false;
  }
  return true;
}

bool TArtAnaFile::AddAnaGateMap(int gate, TArtAnaGate anagate)
{
  if(gate == 0){
    std::cout << "cannot use gate number 0" << std::endl;
    return false;
  }

  std::pair<std::map<int, TArtAnaGate>::iterator, bool> ret
    = fAnaGateMap.insert(std::pair<int, TArtAnaGate>(gate, anagate));
  if(!ret.second){
    std::cout << "conflict gate number: "<< gate << std::endl;
    return false;
  }
  return true;
}

bool TArtAnaFile::AddSequenceMap(std::string name, FSequence fp)
{
  std::pair<std::map<std::string, FSequence>::iterator, bool> ret
    = fSequenceMap.insert(std::pair<std::string, FSequence>(name, fp));
  if(!ret.second){
    std::cout << "sequence "<< name << " already exists" << std::endl;
    return false;
  }
  return true;
}

bool TArtAnaFile::Hist(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues(str, countline, x)) return false;
  if(x.size() != 8 &&
     x.size() !=15){
    std::cout << "number of \',\' is wrong at line: " << countline << std::endl;
    return false;
  }

  std::string title;
  if(!ReadTitle(str, countline, title)) return false;

  std::ostringstream oss_name;
  oss_name << "h" << fHistNameCounter++;

  TH1* hist;
  if(x.size() == 8){//1d hist
    int ii=-1;
    int gate = (int)x[++ii];
    int analyserx = (int)x[++ii];
    int indexbx = (int)x[++ii];
    int indexex = (int)x[++ii];
    int wnumx = (int)x[++ii];
    Int_t nbinsx = (int)x[++ii];
    Double_t xlow = x[++ii];
    Double_t xup = x[++ii];

    std::vector<TArtValIndex> valindexarray;
    for(int loop=0; loop<1+indexex-indexbx; ++loop){
      TArtValIndex x(analyserx, wnumx, indexbx+loop);
      valindexarray.push_back(x);
    }
    if(!AddValIndexMap(oss_name.str(), valindexarray)) return false;
    if(!AddGateMap(oss_name.str(), gate)) return false;
    hist = new TH1D(oss_name.str().c_str(), title.c_str(), nbinsx, xlow, xup);
  }else if(x.size() == 15){//2d hist
    int ii=-1;
    int gate = (int)x[++ii];
    int analyserx = (int)x[++ii];
    int indexbx = (int)x[++ii];
    int indexex = (int)x[++ii];
    int wnumx = (int)x[++ii];
    Int_t nbinsx = (int)x[++ii];
    Double_t xlow = x[++ii];
    Double_t xup = x[++ii];
      
    int analysery = (int)x[++ii];
    int indexby = (int)x[++ii];
    int indexey = (int)x[++ii];
    int wnumy = (int)x[++ii];
    Int_t nbinsy = (int)x[++ii];
    Double_t ylow = x[++ii];
    Double_t yup = x[++ii];

    if(indexex-indexbx != indexey-indexby){
      std::cout << "number of index loop is wrong at line: "
		<< countline << std::endl;
      return false;
    }
    std::vector<TArtValIndex> valindexarray;
    for(int loop=0; loop<1+indexex-indexbx; ++loop){
      TArtValIndex x(analyserx, wnumx, indexbx+loop);
      TArtValIndex y(analysery, wnumy, indexby+loop);
      valindexarray.push_back(x);
      valindexarray.push_back(y);
    }
    if(!AddValIndexMap(oss_name.str(), valindexarray)) return false;
    if(!AddGateMap(oss_name.str(), gate)) return false;
    hist = new TH2D(oss_name.str().c_str(), title.c_str(), nbinsx, xlow, xup,
		    nbinsy, ylow, yup);
  }
  fH1Array.push_back(hist);

  return true;
}

bool TArtAnaFile::And(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues2(str, countline, x)) return false;  
  if(x.size() < 2){
    std::cout << "number of \',\' is too small at line: "
	      << countline << std::endl;
    return false;
  }

  int gate = (int)x[0];
  std::vector<int> andgatearray;
  for(int i=1; i<(int)x.size(); ++i) andgatearray.push_back((int)x[i]);

  if(!AddAnaGateMap(gate, TArtAnaGate(gate, true, false, andgatearray))) return false;
  if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, true, false, andgatearray))) return false;

  return true;
}

bool TArtAnaFile::Or(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues2(str, countline, x)) return false;  
  if(x.size() < 2){
    std::cout << "number of \',\' is too small at line: "
	      << countline << std::endl;
    return false;
  }

  int gate = (int)x[0];
  std::vector<int> andgatearray;
  for(int i=1; i<(int)x.size(); ++i) andgatearray.push_back((int)x[i]);

  if(!AddAnaGateMap(gate, TArtAnaGate(gate, false, true, andgatearray))) return false;
  if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, false, true, andgatearray))) return false;

  return true;
}

bool TArtAnaFile::Gate(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues2(str, countline, x)) return false;  
  if(x.size() != 7){
    std::cout << "number of \',\' is wrong at line: " << countline << std::endl;
    return false;
  }

  int ii=-1;
  int gate = (int)x[++ii];
  int analyser = (int)x[++ii];
  int indexb = (int)x[++ii];
  int indexe = (int)x[++ii];
  int wnum = (int)x[++ii];
  Double_t low = x[++ii];
  Double_t up = x[++ii];

  std::vector<TArtValIndex> valindexarray;
  for(int loop=0; loop<1+indexe-indexb; ++loop){
    TArtValIndex x(analyser, wnum, indexb+loop);
    valindexarray.push_back(x);
  }
  if(!AddValIndexArrayMap(gate, valindexarray)) return false;
  if(!AddValIndexArrayMap(-gate, valindexarray)) return false;
  if(!AddAnaGateMap(gate, TArtAnaGate(gate, low, up))) return false;
  if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, low, up))) return false;

  return true;
}

bool TArtAnaFile::CC(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues2(str, countline, x)) return false;  
  if(x.size() != 13){
    std::cout << "number of \',\' is wrong at line: " << countline << std::endl;
    return false;
  }

  int ii=-1;
  int gate = (int)x[++ii];
  int analyserx = (int)x[++ii];
  int indexbx = (int)x[++ii];
  int indexex = (int)x[++ii];
  int wnumx = (int)x[++ii];
      
  int analysery = (int)x[++ii];
  int indexby = (int)x[++ii];
  int indexey = (int)x[++ii];
  int wnumy = (int)x[++ii];

  Double_t centerx = x[++ii];
  Double_t centery = x[++ii];
  Double_t radiusx = x[++ii];
  Double_t radiusy = x[++ii];

  if(indexex-indexbx != indexey-indexby){
    std::cout << "number of index loop is wrong at line: "
	      << countline << std::endl;
    return false;
  }
  std::vector<TArtValIndex> valindexarray;
  for(int loop=0; loop<1+indexex-indexbx; ++loop){
    TArtValIndex x(analyserx, wnumx, indexbx+loop);
    TArtValIndex y(analysery, wnumy, indexby+loop);
    valindexarray.push_back(x);
    valindexarray.push_back(y);
  }
  if(!AddValIndexArrayMap(gate, valindexarray)) return false;
  if(!AddValIndexArrayMap(-gate, valindexarray)) return false;
  if(!AddAnaGateMap(gate, TArtAnaGate(gate, centerx, centery, radiusx, radiusy))) return false;
  if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, centerx, centery, radiusx, radiusy))) return false;

  return true;
}

bool TArtAnaFile::CutG(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues2(str, countline, x)) return false;  
  if(x.size() != 9){
    std::cout << "number of \',\' is wrong at line: " << countline << std::endl;
    return false;
  }

  std::string name;
  if(!ReadName(str, countline, name)) return false;

  std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.find((int)x[0]);
  if(it != fAnaGateMap.end()){
    std::cout << "gate id " << x[0] << " already exists" << std::endl;	  
    return false;
  }
  TCutG* cutg = new TCutG;
  cutg->SetName(name.c_str());

  int ii=-1;
  int gate = (int)x[++ii];
  int analyserx = (int)x[++ii];
  int indexbx = (int)x[++ii];
  int indexex = (int)x[++ii];
  int wnumx = (int)x[++ii];

  int analysery = (int)x[++ii];
  int indexby = (int)x[++ii];
  int indexey = (int)x[++ii];
  int wnumy = (int)x[++ii];

  if(indexex-indexbx != indexey-indexby){
    std::cout << "number of index loop is wrong at line: "
	      << countline << std::endl;
    return false;
  }
  std::vector<TArtValIndex> valindexarray;
  for(int loop=0; loop<1+indexex-indexbx; ++loop){
    TArtValIndex x(analyserx, wnumx, indexbx+loop);
    TArtValIndex y(analysery, wnumy, indexby+loop);
    valindexarray.push_back(x);
    valindexarray.push_back(y);
  }

  if(!AddValIndexArrayMap(gate, valindexarray)) return false;
  if(!AddValIndexArrayMap(-gate, valindexarray)) return false;
  if(!AddAnaGateMap(gate, TArtAnaGate(gate, cutg))) return false;
  if(!AddAnaGateMap(-gate, TArtAnaGate(-gate, cutg))) return false;

  return true;
}

bool TArtAnaFile::Point(std::string str, int countline)
{
  std::vector<double> x;
  if(!ReadValues2(str, countline, x)) return false;  
  if(x.size() != 3){
    std::cout << "number of \',\' is wrong at line: " << countline << std::endl;
    return false;
  }

  int gate = (int)x[0];
  std::map<int, TArtAnaGate>::iterator it = fAnaGateMap.find(gate);
  if(it == fAnaGateMap.end()){
    std::cout << "gate: " << gate << " does not exist." << std::endl;
    return false;
  }
  TCutG* cutg = it->second.fCutG;
  cutg->SetPoint(cutg->GetN(), x[1], x[2]);

  return true;
}

bool TArtAnaFile::Include(std::string str, int countline)
{
  std::string filename;
  if(!ReadName(str, countline, filename)) return false;
  if(!Open(filename.c_str())){
    std::cout << "include error at line: " << countline << std::endl;
    return false;
  }

  return true;
}

bool TArtAnaFile::ReadSequence(std::string str, int countline, std::string& sequence)
{
  std::istringstream iss(str);
  std::string str2;
  while(1){
    std::getline(iss, str2, '<');
    if(iss.eof()) break;
    std::getline(iss, str2, '>');
    if(iss.eof()){
      std::cout << "sequence sequence is wrong at line: " << countline << std::endl;	  
      return false;
    }
    sequence = str2;
  }
  return true;
}

bool TArtAnaFile::ReadValues(std::string str, int countline, std::vector<double>& x)
{
  std::istringstream iss(str);
  double xtemp;
  std::string str2;
  while(1){
    std::getline(iss, str2, ',');
    if(iss.eof()) break;
    std::istringstream iss2(str2);
    iss2 >> xtemp;
    x.push_back(xtemp);
  }
  return true;
}

bool TArtAnaFile::ReadValues2(std::string str, int countline, std::vector<double>& x)
{
  std::istringstream iss(str);
  double xtemp;
  std::string str2;
  std::getline(iss, str2, '>');
  while(1){
    std::getline(iss, str2, ',');
    if(iss.eof()) break;
    std::istringstream iss2(str2);
    iss2 >> xtemp;
    x.push_back(xtemp);
  }
  return true;
}

bool TArtAnaFile::ReadName(std::string str, int countline, std::string& name)
{
  std::istringstream iss(str);
  std::string str2;
  std::getline(iss, str2, '"');
  if(iss.eof()) return false;
  std::getline(iss, str2, '"');
  if(iss.eof()){
    std::cout << "number of \'\"\' is wrong at line: " << countline << std::endl;	  
    return false;
  }
  name = str2;
  return true;
}

bool TArtAnaFile::ReadTitle(std::string str, int countline, std::string& title)
{
  std::istringstream iss(str);
  std::string str2;
  while(1){
    std::getline(iss, str2, '"');
    if(iss.eof()) break;
    std::getline(iss, str2, '"');
    if(iss.eof()){
      std::cout << "number of \'\"\' is wrong at line: " << countline << std::endl;	  
      return false;
    }
    title = str2;
  }
  return true;
}
